/* stock set of macros and declarations for Antelope programs danq 6/5/92 */
/* $Name $Revision: 1.107 $ $Date: 2005/06/16 15:46:56 $ */

#ifndef _stock_
#define _stock_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "deviants_include.h"

#include <malloc.h>
#include <sys/stat.h>
#include H_STATVFS 

#include <netdb.h>
#include <sys/types.h>
#include <inttypes.h>
#include <regex.h>

#include "deviants.h"
#include "stocktime.h"

#define strtod(A,B) fix_strtod(A,B)
 
#include "arrays.h"

#define BITS_PER_INT   	(8*sizeof(int))

#define bitnew() inittbl(0,1,0,0,sizeof(char *))
#define bitfree(B) freetbl(B,0)
#define bitempty(B) clrtbl(B,0) 

#define HOOK_MAGIC 814615491

typedef struct Hook {
    int magic ; 
    void (*free)() ;
    int type ;
    void *p ;
} Hook ; 

typedef struct Xlat {
        char *name ;
        int num ;
} Xlat ;

typedef struct Submap { 
    char *pattern ; 
    regex_t prog ; 
    char *substitution ; 
} Submap ;

typedef Tbl Morphtbl ;

#define MORPH_PARTIAL 1
#define MORPH_ALL 2

typedef struct InfoPath {
    unsigned long 	device ;		/* device id */
    char	host[MAXHOSTNAMELEN] ;  /* host: null if localhost */
    char        fs[FILENAME_MAX] ;	/* filesystem mount point */
    unsigned long	avail ; 		/* # bytes available */
    unsigned long	total ;			/* # bytes total */
    unsigned long	avail_inodes ; 		/* # inodes available */
    unsigned long	total_inodes ;		/* # inodes total */
    char	fstype[32] ;	/* filesystem type */
    int		max_filename ;		/* maximum path length ? */
    int		flags ;			/* bit 0 (==1) is set when mounted read-only */
    char	fstr[32] ; 
} InfoPath ; 

#define STASH "\377"
#define STASHC 0xff
#define STASH_LENGTH 1
#define is_stash(SRCNAME,BUFFER,NBYTES) (((NBYTES)>0)&&(BUFFER[0]&0xff) == STASHC)

#define SG_PORT 56789
#define SG_MAX_SIGNATURE 512

#define FS_DEPTH_FIRST 4
#define FS_ONLY_DIR 2
#define FS_ONLY_FILE 1

#include "elog.h"
#include "pf.h"
#include "bury.h"

#ifdef  __cplusplus
extern "C" {
#endif

extern double fix_strtod ( const char *in, char **end ) ;

extern Hook *new_hook ( void (*hookfree)(void *p) );
extern void free_hook ( Hook **hookp );

extern char **antelope_glob ( char *v );
extern char *ask ( char *fmt, ... );
extern char *asknoecho ( char *fmt, ... );
extern int askyn ( char *fmt, ... );
extern int askynaq ( char *fmt, ... );

extern int tty_noecho ( int fd );
extern int tty_echo ( int fd );
extern int tty_immediate ( int fd );
extern int tty_byline ( int fd );
extern int tty_reset ( int fd );

extern char * getantelope ( void );
extern int newdata ( char *returnpath, char *envname, char *dirname, char *filename, char *suffix );

extern Tbl * datasearch ( char *path, char *dirname, char *filename, int flags );
extern char * datapath ( char *envname, char *dirname, char *filename, char *suffix );
extern char * datafile ( char *envname, char *filename );

extern char *newcs ( char *f_s, int n_s ) ;
extern char * expand_env ( char *s );

extern int nstrcmp(char *a, char *b);
extern Tbl *split ( char *s, int c );
extern int gz_openfd ( char *filename );
extern FILE *gz_open ( char *filename );
extern FILE *gz_cat ( char *filename );
extern int blank ( char *s );
extern void dirbase ( char *path, char *dir, char *base );
extern int fixup_env ( char *envsetup[], int nenv, int force );
extern int envfile ( char *filename );
extern int gethdir ( char *user, char *home );
extern void isort ( char *a, int n, int size, int (*compare) (char *, char*, void *), void *pvt );

extern char * homedir ( char *user );
extern int cleanpath ( char *, int, char *);
extern char *concatpaths ( char *s0, ... );
extern int abspath ( char *relp, char *absp );
extern int relpath ( char *from, char *to, char *relp );
extern void parsepath ( char *path, char *dir, char *base, char *suffix );
extern int fspath ( char *path, char *host, char *fs );
extern struct InfoPath * infopath ( char *path );
extern int samepath(char *path1, char *path2) ;

extern int make_pathdirs ( char *filename );
extern int makedir ( char *dir );
extern int mappath ( char *spath, char *upath );
extern void quicksort ( char *pbase, size_t total_elems, size_t size, int (*cmp)(), void *);
extern void shellsort ( char *a, int n, int size, int (*compare)(), void *);
extern int ssearch ( char *key, int keysize, char *index, int nkeys, int unique, int (*compare)(), void *, int *ns, int *ne );

extern int strmatches ( char *s, char *pattern, Hook **hookp );
extern int strcontains ( char *s, char *pattern, Hook **hookp, int *start, int *nchars );
extern char *strdbl ( double x, char *buf );

extern int blank ( char *s );
extern int whitespace ( char *s );
extern Tbl *split ( char *s, int c );
extern void sncopy ( char *dest, char *source, int n );
extern void szcopy ( char *dest, char *source, int n );
extern void copystrip ( char *dest, char *source, int n );
extern void * memdup ( void *a, int n );
extern void str2upper ( char *s );
extern void str2lower ( char *s );
extern char * strconcat ( char *s, ... );
extern void strtrim ( char *s );
extern void strsub ( char *oldstr, char *pattern, char *replacement, char *newstr );
extern int pushstr ( void **vstack, char *s );
extern int pushstrc ( void **vstack, int c ) ;
extern int pushstrn ( void **vstack, char *s, int l );
extern char *popstr ( void **vstack, int vfree );
extern void strtr ( char *s, char *from, char *to );
extern char *getaline( FILE *file, char *aline, int n );
extern char *intactline ( FILE *file, char **aline, int *size );
extern void xargs ( char *cmd, Tbl *args, char *tail, int verbose );
extern void chomp (char *s) ;
extern double sz2dbl(char *size) ;

#define gets(ALINE) getaline(stdin, ALINE, sizeof(ALINE))

extern int bitset ( Bitvector *bvec, int index );
extern int bitclr ( Bitvector *bvec, int index );
extern int bittst ( Bitvector *bvec, int index );
extern int bitmax ( Bitvector *b );
extern int bitcmp ( Bitvector *b1, Bitvector *b2 );

#ifndef	__cplusplus
extern Bitvector * bitand ( Bitvector *b1, Bitvector *b2 );
extern Bitvector * bitor ( Bitvector *b1, Bitvector *b2 );
#endif

extern Bitvector * bitnot ( Bitvector *b );
extern Bitvector * bitxor ( Bitvector *b1, Bitvector *b2 );

extern void setabit ( int *flag, int abit );
extern void clrabit ( int *flag, int abit );

extern int fdkey ( int fd );
extern int fdwait ( int fd, int msec );

extern char * aprintchar ( int c );
extern void aprintstr ( FILE *file, char *s, int n );
extern char * hexdump_string ( long base, void *memory, int l );
extern void Hexdump ( FILE *file, long base, void *memory, int l );
extern void hexdump ( FILE *file, void *memory, int l );
extern void printable_string ( char *buf, int n, char *memory, int l );
extern void asciidump ( FILE *file, char *memory, int l );
extern int hex2int ( char c );
extern int read_asciidump ( FILE *file, char *memory, int l );
extern int read_hexdump ( FILE *file, char *memory, int l );
extern int hex2mem ( char *hex, void *mem );
extern void mem2hex ( void *mem, int n, char *s );
extern int printable ( char *s, int n );
extern void Xchardump ( FILE *file, char *label, char *desc, char *bytes, int nbytes );
extern void showdouble ( FILE *file, double x );
extern void showfloat ( FILE *file, float x );

extern void banner ( char *program, char *version );
extern void cbanner ( char *version, char *usage, char *author, char *location, char *email );

extern int gencompress ( unsigned char **out, int *nout, int *size, int *in, int nsamp, int length );
extern int genuncompress ( int **out, int *nout, int *size, unsigned char *in, int nbytes );

extern int uucsd ( int *data, char **bufp, int bufcnt, int nsamp );
extern int cucsd ( int *data, int nsamp, char **buf, int *nbytes, int *bufsiz );

extern int ccanada ( int *data, int nsamp, char **compressedp, int *nbytes, int *compressedsz );
extern int ucanada ( char *compressed, int nbytes, int **datap, int *nsamp, int *datasz );

extern void gse_checksum ( int *signal, int nsamp, int *checksum );
extern int cm6 ( int *data, int nsamp, char **compressedp, int *nbytes, int *compressedsz );
extern int um6 ( char *compressed, int nbytes, int **datap, int *nsamp, int *datasz );

extern char * xlatnum ( int def, Xlat *xlat, int nxlat );
extern int xlatname ( char *name, Xlat *xlat, int nxlat );

extern int runcmd ( char *argv[], char **result );
extern int verbose_exists (void) ;

extern int set_random ( int intcount );
extern void fill_random ( char *packet, int nchar, int *pp );
extern int check_random ( char *packet, int nchar );

extern void ignoreSIGPIPE ( void );
extern void * usermethod ( char *methodname );
extern int set_fl ( int fd, int flags );
extern int clr_fl ( int fd, int flags );
extern int yesno ( char *s );


typedef struct Mapdata {
    char           *mmap_data;
    int             mmap_size;
    int             counter ;
    int		    ref ;
}               Mapdata;

extern Mapdata * vmapfile ( char *filename, Arr ** maparr );
extern int mapfile ( char *filename, Arr ** maparr, char **mmap_datap );
extern int unmapfile ( char *filename, Arr ** maparr );
extern int unmaparr ( Arr ** maparr );
extern int wmapfile ( char *filename, Arr ** maparr, char **mmap_datap );

extern int check_cmd ( char *command, char *pathenv );
extern int my_system ( char *command, int share, int *coreflag, int *termsig );
extern char * parse_on_1char ( char *string, char parse_char1 );

extern void qtcrc ( int *crcp, unsigned char *buffer, int bufsize );
extern void qtchecksum ( short int *checksump, unsigned char *buffer, int bufsize );
extern int userauth ( char *user );
extern int xuserauth ( char *user );

extern int mkfile ( char *name, unsigned long nbytes, char *init, int initsize );
extern void regsubstitute ( char *, char *, regmatch_t *, char *);
extern int patsub ( char *, regex_t *, char *, char *);

extern void freemorphtbl ( Morphtbl *morphmap );
extern int newmorphtbl ( Tbl *list, Morphtbl **morphmap );
extern int morphtbl ( char *old, Morphtbl *morphmap, int flags, char *result );

extern double an_infinity ( void );
extern double a_signaling_nan ( void );
extern double a_quiet_nan ( void );
extern double a_nan ( void );
extern int is_infinity ( double x );
extern int is_negative_infinity ( double x );
#define is_nan(X)    isnan((X))

extern int is_file ( char *path );
extern int is_dir ( char *path );
extern int is_present ( char *path );
extern int is_empty ( char *path ) ;
extern unsigned int filesize ( char *path );
extern unsigned int filemode ( char *path );
extern Tbl * dirfiles ( char *path, int flag );
extern int fstraverse ( char *path, int flags, int (*user)(char *, struct stat *, void *), void *pvt );

extern int is_tty ( int fd );
extern int tty_noecho ( int fd );
extern int tty_echo ( int fd );
extern int tty_immediate ( int fd );
extern int tty_byline ( int fd );
extern int tty_reset ( int fd );

#define RB_IDSZ 64
#define RB_MAGIC  "ringbuff"
typedef struct RbData { 
    double t ; 
    double v ; 
} RbData ; 

typedef struct RbInfo { 
    char magic[8] ;
    char id[RB_IDSZ] ; 
    int p, cnt, max, flags ; 
} RbInfo ; 

typedef struct RingBuffer { 
    RbInfo *info ; 
    RbData *data ;
    size_t size ;
    int readonly ;
} RingBuffer ;

extern int newring ( char *filename, char *id, int max, int flags );
extern RingBuffer * openring ( char *filename, char *id, int max, int flags );
extern void closering ( RingBuffer *rb );
extern int pushring ( RingBuffer *rb, double time, double value );
extern int countring ( RingBuffer *rb );
extern int getring ( RingBuffer *rb, int i, double *time, double *value );
extern int findring ( RingBuffer *rb, double value, double deltat );
extern double maxring ( RingBuffer *rb, double deltat, int *index );
extern double minring ( RingBuffer *rb, double deltat, int *index );
extern double avgring ( RingBuffer *rb, double deltat );
extern int changedring ( RingBuffer *rb, double deltat );

extern void closeringlet ( void *rb );
extern int newringlet ( char *filename, int max, int entrysize );
extern void * openringlet ( char *filename, int max, int entrysize );
extern int pushringlet ( void *rb, char *entry );
extern int countringlet ( void *rb );
extern int getringlet ( void *rb, int i, char **entry );

extern double disordered ( void );
extern double r_disordered (Hook **);

extern void showenv ( FILE *file );

/*
extern int is_stash ( char *srcname, char *buffer, int nbytes) ;
*/
extern char *machine_info(int flag) ; 

extern int legit_ip_pf ( Pf *pf, unsigned int ip, char **rstrict );
extern int legit_ip ( char *pfname, unsigned int ip, char **rstrict );

extern int lfit ( double *x, double *y, int npts, double *slope, double *intercept );
extern void lchi ( double *x, double *y, int npts, double slope, double intercept, double *ymaxdev, double *chisqr );

extern double dround ( double x );
extern double d4round ( double x );

extern int units_convert ( double in, char *in_units, char *want, double *out, char *out_units );
extern Tbl * units_match ( char *in_units );

extern int sgsign ( char *signer, char *data, int nbytes, char *signature );
extern int sgverify ( char *signer, char *data, int nbytes, char *signature );

extern double findmin ( double a, double b, double (*f) (double, void *), double tol, void *p );
extern double findmax ( double a, double b, double (*f) (double, void *), double tol, void *p );
extern double findzero ( double ax, double bx, double (*f) (double, void *), double tol, void *p );

extern int restat ( const char *path, struct stat *buf );
extern int relstat ( const char *path, struct stat *buf );
extern int refstat ( int fd, struct stat *buf );

#ifdef	__cplusplus
}
#endif

/* When a macro generates multiple output statements, there is a danger
 * that these statements might lead to odd results in a context where a
 * single statement is expected, eg. after an if -- STMT forces the macro
 * output to be a single statement, and the following stuff with ZERO is
 * to make lint work properly. */

#ifdef lint
extern int      ZERO;
#else
#define ZERO 0
#endif

#define STMT(stuff) do { stuff } while (ZERO)

/* The following macros simplify memory allocation and testing
 *
 * allot and reallot are convenient interfaces to malloc and realloc which
 * perform testing on the result without cluttering the code with
 * branches and messages which are seldom if ever used.
 *
 * ALLOTERROR specifies what happens when a malloc or realloc fails -- it
 * may be overridden with a special macro within the file */

#ifdef __STDC__
#define ALLOTDIE(ptr,size)	die(1,"Out of memory at line %d in '%s'\n\tCan't malloc %u bytes for " #ptr "\n", __LINE__, __FILE__, size)
#else
#define ALLOTDIE(ptr,size)	die(1,"Out of memory at line %d in '%s'\n\tCan't malloc %u bytes for ptr\n", __LINE__, __FILE__, size)
#endif

#ifndef BADALLOT
#define BADALLOT -1
#endif

#ifdef lint
extern int ReTuRn(int) ;
#define BADRETURN ReTuRn(BADALLOT)
#else
#define BADRETURN return BADALLOT
#endif

#ifdef __STDC__
#define ALLOTREGISTER_ERROR(ptr,size) \
    STMT(register_error(1,"Out of memory at line %d in '%s'\n\tCan't malloc %u bytes for " #ptr "\n", __LINE__, __FILE__, size); BADRETURN ;)
#else
#define ALLOTREGISTER_ERROR(ptr,size) \
    STMT(register_error(1,"Out of memory at line %d in '%s'\n\tCan't malloc %u bytes for ptr\n", __LINE__, __FILE__, size); BADRETURN ;)
#endif

#ifdef __STDC__
#define ALLOTCOMPLAIN(ptr,size) \
    {complain(1,"Out of memory at line %d in '%s'\n\tCan't malloc %u bytes for " #ptr "\n", __LINE__, __FILE__, size); BADRETURN ;}
#else
#define ALLOTCOMPLAIN(ptr,size) \
    {complain(1,"Out of memory at line %d in '%s'\n\tCan't malloc %u bytes for ptr\n", __LINE__, __FILE__, size); BADRETURN ;}
#endif

#ifndef ALLOTERROR
#define ALLOTERROR ALLOTDIE
#endif

#define SIZE_BUFFER(TYPE,BUFFER,BUFSZ,NEEDED) \
    if ((NEEDED)>(BUFSZ)) { \
	if ((BUFSZ) != 0) { \
	    free(BUFFER) ; \
	} \
	allot(TYPE,BUFFER,NEEDED) ; \
	BUFSZ = NEEDED ; \
    }

#define CSIZE_BUFFER(TYPE,BUFFER,BUFSZ,NEEDED) \
    if ((NEEDED)>(BUFSZ)) { \
	if ((BUFSZ) != 0) { \
	    free(BUFFER) ; \
	} \
	callot(TYPE,BUFFER,NEEDED) ; \
	BUFSZ = NEEDED ; \
    }

#define RESIZE_BUFFER(TYPE,BUFFER,BUFSZ,NEEDED) \
    if ((NEEDED)>(BUFSZ)) { \
	if ((BUFSZ) != 0) { \
	    reallot(TYPE,BUFFER,NEEDED) ; \
	} else { \
	    allot(TYPE,BUFFER,NEEDED) ; \
	} \
	BUFSZ = NEEDED ; \
    }


/* allotted can be used in an if statement when the action in allot is
 * inadequate */
#define allotted(type,ptr,size) \
  ( (ptr=(type) malloc((unsigned)((size)*sizeof(*(ptr))))) != NULL )
#define callotted(type,ptr,size) \
  ( (ptr=(type) calloc((unsigned)(size),sizeof(*(ptr))) ) != NULL )

/* allot checks the results of malloc and generates an error message in
 * the case of failures */
#define allot(type,ptr,size)    \
  STMT( if (!allotted(type,ptr,size)) ALLOTERROR(ptr,size);)
#define callot(type,ptr,size)    \
  STMT( if (!callotted(type,ptr,size)) ALLOTERROR(ptr,size);)

/* reallot and reallotted correspond to allot and allotted and are
 * interfaces to realloc. */
#define reallotted(type,ptr,size) \
  ( (ptr=(type) realloc((char *)ptr,(unsigned)((size)*sizeof(*ptr)))) != NULL )
#define reallot(type,ptr,size)    \
  STMT( if (!reallotted(type,ptr,size)) ALLOTERROR(ptr,size);)


/* insist provides a concise method for consistency checks which are
 * never expect to fail, avoiding cluttering the code with branches which
 * are seldom if ever taken.
 * 
 * INSISTFAIL specifies what happens when an insist fails -- it may be
 * overridden with a special macro within a file */

#ifndef INSISTFAIL
#define INSISTFAIL die
#endif

#define insist(ex)	STMT( if (!(ex)){ INSISTFAIL(1, "Unexpected failure: file %s, line %d\n", __FILE__, __LINE__ ) ;} )
#define warn(ex)	STMT( if (!(ex)){ elog_debug(1, "Unexpected failure: file %s, line %d\n", __FILE__, __LINE__ ) ;} )


/* The meaning of the following macros is fairly obvious; however, they
 * are all dangerous in the sense that their arguments will be evaluated
 * multiple times, which can lead to erroneous results. for example,
 * sqr(a++) will give an undefined result which is probably not what you
 * wanted and may differ among compilers. */

#define sqr(d) 		((d)*(d))

#ifndef SQR
#define SQR(d) 		((d)*(d))
#endif

#ifndef ABS
#define ABS(d) 		((d)< 0  ? -(d) : (d))
#endif

#ifndef MAX
#define MAX(a,b)        ((a)>(b) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a,b)        ((a)<(b) ? (a) : (b))
#endif

#ifndef SIGN
#define SIGN(d) 	(((d)>0)? 1.0 : ((d)<0) ? -1.0 : 0.0)
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/* The following size is judged to be large enough to avoid problems with
 * overflowing string buffers, so that the various string routines which
 * do no checking can be used without much fear. */
#define STRSZ 1024

#endif


/* $Id: stock.h,v 1.107 2005/06/16 15:46:56 danq Exp $ */
